---------------------------------------------------------------------------
--
-- Copyright (c) 2016
-- Mitsubishi Electric Corporation
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are
-- met:
-- 
-- 1. Redistributions of source code must retain the above copyright
-- notice, this list of conditions and the following disclaimer.
-- 
-- 2. Redistributions in binary form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-- THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-- OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- File Name: MA_LFSR_update.vhd
-- Type: RTL 
-- Author: Takeshi SUGAWARA
-- Date: 2016/05/26
-- Version: 1.0
-- Abstract:
-- Modification History: 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;  

entity MA_LFSR_update is
  port(
    Din:    in  std_logic_vector(255 downto 0);
    mode:   in  std_logic;
    Dout:   out std_logic_vector(255 downto 0);
    times2: out std_logic_vector(255 downto 0);
    times3: out std_logic_vector(255 downto 0)
    );
end MA_LFSR_update;

architecture RTL of MA_LFSR_update is
  component MA_LFSR_func256 is
    port(
      x: in  std_logic_vector(255 downto 0);
      y: out std_logic_vector(255 downto 0)
      );
  end component;
  
  constant P_MODE_X2: std_logic := '0';
  constant P_MODE_X3: std_logic := '1';  

  signal L_times2: std_logic_vector(255 downto 0);
  signal L_times3: std_logic_vector(255 downto 0);  
begin

  gf256: MA_LFSR_func256 port map(
    x => Din,
    y => L_times2);

  L_times3 <= Din xor L_times2;

  Dout <= L_times2 when (mode = P_MODE_X2) else
          L_times3;

  times2 <= L_times2;
  times3 <= L_times3;  
end;
