---------------------------------------------------------------------------
--
-- Copyright (c) 2016
-- Mitsubishi Electric Corporation
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are
-- met:
-- 
-- 1. Redistributions of source code must retain the above copyright
-- notice, this list of conditions and the following disclaimer.
-- 
-- 2. Redistributions in binary form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-- THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-- OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- File Name: MA_Sbox128.vhd
-- Type: RTL 
-- Author: Takeshi SUGAWARA
-- Date: 2016/05/26
-- Version: 1.0
-- Abstract:
-- Modification History: 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;  

entity MA_Sbox128 is
  port(
    x: in  std_logic_vector(127 downto 0);
    y: out std_logic_vector(127 downto 0)
    );
end MA_Sbox128;

architecture RTL of MA_Sbox128 is
  component MA_Sbox is
    port(
      x: in  std_logic_vector(3 downto 0);
      y: out std_logic_vector(3 downto 0)
      );
  end component;
  
  signal x00, x01, x02, x03: std_logic_vector(3 downto 0); 
  signal x04, x05, x06, x07: std_logic_vector(3 downto 0); 
  signal x08, x09, x10, x11: std_logic_vector(3 downto 0); 
  signal x12, x13, x14, x15: std_logic_vector(3 downto 0); 
  signal x16, x17, x18, x19: std_logic_vector(3 downto 0); 
  signal x20, x21, x22, x23: std_logic_vector(3 downto 0); 
  signal x24, x25, x26, x27: std_logic_vector(3 downto 0);
  signal x28, x29, x30, x31: std_logic_vector(3 downto 0);

  signal y00, y01, y02, y03: std_logic_vector(3 downto 0);
  signal y04, y05, y06, y07: std_logic_vector(3 downto 0);   
  signal y08, y09, y10, y11: std_logic_vector(3 downto 0); 
  signal y12, y13, y14, y15: std_logic_vector(3 downto 0); 
  signal y16, y17, y18, y19: std_logic_vector(3 downto 0); 
  signal y20, y21, y22, y23: std_logic_vector(3 downto 0); 
  signal y24, y25, y26, y27: std_logic_vector(3 downto 0);
  signal y28, y29, y30, y31: std_logic_vector(3 downto 0);
begin
  
  x00 <= x(127 downto 124);
  x01 <= x(123 downto 120);
  x02 <= x(119 downto 116);
  x03 <= x(115 downto 112);
  x04 <= x(111 downto 108);
  x05 <= x(107 downto 104);
  x06 <= x(103 downto 100);
  x07 <= x( 99 downto  96);
  x08 <= x( 95 downto  92);
  x09 <= x( 91 downto  88);
  x10 <= x( 87 downto  84);
  x11 <= x( 83 downto  80);
  x12 <= x( 79 downto  76);
  x13 <= x( 75 downto  72);
  x14 <= x( 71 downto  68);
  x15 <= x( 67 downto  64);
  x16 <= x( 63 downto  60);
  x17 <= x( 59 downto  56);
  x18 <= x( 55 downto  52);
  x19 <= x( 51 downto  48);
  x20 <= x( 47 downto  44);
  x21 <= x( 43 downto  40);
  x22 <= x( 39 downto  36);
  x23 <= x( 35 downto  32);
  x24 <= x( 31 downto  28);
  x25 <= x( 27 downto  24);
  x26 <= x( 23 downto  20);
  x27 <= x( 19 downto  16);
  x28 <= x( 15 downto  12);
  x29 <= x( 11 downto   8);
  x30 <= x(  7 downto   4);
  x31 <= x(  3 downto   0);


  y <= y00 & y01 & y02 & y03 & y04 & y05 & y06 & y07 &
       y08 & y09 & y10 & y11 & y12 & y13 & y14 & y15 &
       y16 & y17 & y18 & y19 & y20 & y21 & y22 & y23 &
       y24 & y25 & y26 & y27 & y28 & y29 & y30 & y31;
  
  s00: MA_Sbox port map ( x => x00, y => y00);
  s01: MA_Sbox port map ( x => x01, y => y01);
  s02: MA_Sbox port map ( x => x02, y => y02);
  s03: MA_Sbox port map ( x => x03, y => y03);
  s04: MA_Sbox port map ( x => x04, y => y04);
  s05: MA_Sbox port map ( x => x05, y => y05);
  s06: MA_Sbox port map ( x => x06, y => y06);
  s07: MA_Sbox port map ( x => x07, y => y07);
  s08: MA_Sbox port map ( x => x08, y => y08);
  s09: MA_Sbox port map ( x => x09, y => y09);
  s10: MA_Sbox port map ( x => x10, y => y10);
  s11: MA_Sbox port map ( x => x11, y => y11);
  s12: MA_Sbox port map ( x => x12, y => y12);
  s13: MA_Sbox port map ( x => x13, y => y13);
  s14: MA_Sbox port map ( x => x14, y => y14);
  s15: MA_Sbox port map ( x => x15, y => y15);
  s16: MA_Sbox port map ( x => x16, y => y16);
  s17: MA_Sbox port map ( x => x17, y => y17);
  s18: MA_Sbox port map ( x => x18, y => y18);
  s19: MA_Sbox port map ( x => x19, y => y19);
  s20: MA_Sbox port map ( x => x20, y => y20);
  s21: MA_Sbox port map ( x => x21, y => y21);
  s22: MA_Sbox port map ( x => x22, y => y22);
  s23: MA_Sbox port map ( x => x23, y => y23);
  s24: MA_Sbox port map ( x => x24, y => y24);
  s25: MA_Sbox port map ( x => x25, y => y25);
  s26: MA_Sbox port map ( x => x26, y => y26);
  s27: MA_Sbox port map ( x => x27, y => y27);
  s28: MA_Sbox port map ( x => x28, y => y28);
  s29: MA_Sbox port map ( x => x29, y => y29);
  s30: MA_Sbox port map ( x => x30, y => y30);
  s31: MA_Sbox port map ( x => x31, y => y31);
end;
  
