---------------------------------------------------------------------------
--
-- Copyright (c) 2016
-- Mitsubishi Electric Corporation
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are
-- met:
-- 
-- 1. Redistributions of source code must retain the above copyright
-- notice, this list of conditions and the following disclaimer.
-- 
-- 2. Redistributions in binary form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-- THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-- OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- File Name: MA_T256.vhd
-- Type: RTL 
-- Author: Takeshi SUGAWARA
-- Date: 2016/05/26
-- Version: 1.0
-- Abstract:
-- Modification History: 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;  

entity MA_T256 is
  port(
    x: in  std_logic_vector(255 downto 0);
    y: out std_logic_vector(255 downto 0)
    );
end MA_T256;

architecture RTL of MA_T256 is
  component MA_sr128 is
    port(
      x: in  std_logic_vector(127 downto 0);
      y: out std_logic_vector(127 downto 0)
      );
  end component;

  component MA_srinv128 is
    port(
      x: in  std_logic_vector(127 downto 0);
      y: out std_logic_vector(127 downto 0)
      );
  end component;
  
  signal x_row_a0: std_logic_vector(31 downto 0);
  signal x_row_a1: std_logic_vector(31 downto 0);
  signal x_row_a2: std_logic_vector(31 downto 0);
  signal x_row_a3: std_logic_vector(31 downto 0);

  signal x_row_b0: std_logic_vector(31 downto 0);
  signal x_row_b1: std_logic_vector(31 downto 0);
  signal x_row_b2: std_logic_vector(31 downto 0);
  signal x_row_b3: std_logic_vector(31 downto 0);

  signal y_row_a0: std_logic_vector(31 downto 0);
  signal y_row_a1: std_logic_vector(31 downto 0);
  signal y_row_a2: std_logic_vector(31 downto 0);
  signal y_row_a3: std_logic_vector(31 downto 0);
  
  signal y_row_b0: std_logic_vector(31 downto 0);
  signal y_row_b1: std_logic_vector(31 downto 0);
  signal y_row_b2: std_logic_vector(31 downto 0);
  signal y_row_b3: std_logic_vector(31 downto 0);
  
  signal xA:       std_logic_vector(127 downto 0);
  signal xB:       std_logic_vector(127 downto 0);  
  signal yA:       std_logic_vector(127 downto 0);
  signal yB:       std_logic_vector(127 downto 0);  
begin
  x_row_a0 <= x(255 downto 224);
  x_row_b0 <= x(223 downto 192);
  x_row_a1 <= x(191 downto 160);
  x_row_b1 <= x(159 downto 128);
  x_row_a2 <= x(127 downto  96);
  x_row_b2 <= x( 95 downto  64);
  x_row_a3 <= x( 63 downto  32);
  x_row_b3 <= x( 31 downto   0);

  xA <= x_row_a0 & x_row_a1 & x_row_a2 & x_row_a3;
  xB <= x_row_b0 & x_row_b1 & x_row_b2 & x_row_b3;

  srX: MA_sr128    port map ( x => xA, y => yA);
  srY: MA_srinv128 port map ( x => xB, y => yB);

  y_row_a0 <= yB(127 downto 96);
  y_row_a1 <= yB( 95 downto 64);
  y_row_a2 <= yB( 63 downto 32);
  y_row_a3 <= yB( 31 downto  0);

  y_row_b0 <= yA(127 downto 96);
  y_row_b1 <= yA( 95 downto 64);
  y_row_b2 <= yA( 63 downto 32);
  y_row_b3 <= yA( 31 downto  0);

  y <= y_row_a0 & y_row_b0 & 
       y_row_a1 & y_row_b1 & 
       y_row_a2 & y_row_b2 & 
       y_row_a3 & y_row_b3;
    
--  y(255 downto 224) <= y_row_a0;
--  y(223 downto 192) <= y_row_b0;
--  y(191 downto 160) <= y_row_a1;
--  y(159 downto 128) <= y_row_b1;
--  y(127 downto  96) <= y_row_a2;
--  y( 95 downto  64) <= y_row_b2;
--  y( 63 downto  32) <= y_row_a3;
--  y( 31 downto   0) <= y_row_b3;
end;
