---------------------------------------------------------------------------
--
-- Copyright (c) 2016
-- Mitsubishi Electric Corporation
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are
-- met:
-- 
-- 1. Redistributions of source code must retain the above copyright
-- notice, this list of conditions and the following disclaimer.
-- 
-- 2. Redistributions in binary form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-- THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-- OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- File Name: MA_const_gen256.vhd
-- Type: RTL 
-- Author: Takeshi SUGAWARA
-- Date: 2016/05/26
-- Version: 1.0
-- Abstract:
-- Modification History: 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;  

entity MA_const_gen256 is
  port(
    rnd:    in  std_logic_vector(17 downto 0);
    encdec: in  std_logic;
    y:      out std_logic_vector(255 downto 0)
    );
end MA_const_gen256;

architecture RTL of MA_const_gen256 is
  constant C_ZERO32: std_logic_vector(31 downto 0) := X"00000000";
  constant C_ZERO16: std_logic_vector(15 downto 0) := X"0000";
  
  signal r0e, r1e, r2e, r3e: std_logic_vector(  3 downto 0);
  signal r0d, r1d, r2d, r3d: std_logic_vector(  3 downto 0);
  signal round_enc:          std_logic_vector(  4 downto 0);
  signal round_dec:          std_logic_vector(  4 downto 0);
  signal rcon_enc, rcon_dec: std_logic_vector(255 downto 0);
begin

  round_enc_decoder: process(rnd)
  begin
    case rnd is
      when "000000000000000010" => round_enc <= "00001";
      when "000000000000000100" => round_enc <= "00010";
      when "000000000000001000" => round_enc <= "00011";
      when "000000000000010000" => round_enc <= "00100";
      when "000000000000100000" => round_enc <= "00101";
      when "000000000001000000" => round_enc <= "00110";
      when "000000000010000000" => round_enc <= "00111";
      when "000000000100000000" => round_enc <= "01000";
      when "000000001000000000" => round_enc <= "01001";
      when "000000010000000000" => round_enc <= "01010";
      when "000000100000000000" => round_enc <= "01011";
      when "000001000000000000" => round_enc <= "01100";
      when "000010000000000000" => round_enc <= "01101";
      when "000100000000000000" => round_enc <= "01110";
      when "001000000000000000" => round_enc <= "01111";
      when "010000000000000000" => round_enc <= "10000";
      when "100000000000000000" => round_enc <= "10001"; 
      when others => round_enc <= "00000";
    end case;
  end process round_enc_decoder;

  round_dec_decoder: process(rnd)
  begin
    case rnd is
      when "000000000000000010" => round_dec <= "01111";
      when "000000000000000100" => round_dec <= "01110";
      when "000000000000001000" => round_dec <= "01101";
      when "000000000000010000" => round_dec <= "01100";
      when "000000000000100000" => round_dec <= "01011";
      when "000000000001000000" => round_dec <= "01010";
      when "000000000010000000" => round_dec <= "01001";
      when "000000000100000000" => round_dec <= "01000";
      when "000000001000000000" => round_dec <= "00111";
      when "000000010000000000" => round_dec <= "00110";
      when "000000100000000000" => round_dec <= "00101";
      when "000001000000000000" => round_dec <= "00100";
      when "000010000000000000" => round_dec <= "00011";
      when "000100000000000000" => round_dec <= "00010";
      when "001000000000000000" => round_dec <= "00001";
      when "010000000000000000" => round_dec <= "00000"; 
      when others => round_dec <= "10000";
    end case;
  end process round_dec_decoder;
  
  r0e <= round_enc(3 downto 0);
  r1e <= round_enc(3 downto 0) xor X"1";
  r2e <= round_enc(3 downto 0) xor X"2";
  r3e <= round_enc(3 downto 0) xor X"3";  

  r0d <= round_dec(3 downto 0);
  r1d <= round_dec(3 downto 0) xor X"1";
  r2d <= round_dec(3 downto 0) xor X"2";
  r3d <= round_dec(3 downto 0) xor X"3";  
  
  rcon_enc <= C_ZERO32 & r0e & r1e & r2e & r3e & C_ZERO16 &
              C_ZERO32 & r1e & r0e & r3e & r2e & C_ZERO16 &
              C_ZERO32 & r2e & r3e & r0e & r1e & C_ZERO16 &
              C_ZERO32 & r3e & r2e & r1e & r0e & C_ZERO16;

  rcon_dec <= C_ZERO32 & r2d & r3d & r0d & r1d & C_ZERO16 &
              C_ZERO32 & r3d & r2d & r1d & r0d & C_ZERO16 &
              C_ZERO32 & r0d & r1d & r2d & r3d & C_ZERO16 &
              C_ZERO32 & r1d & r0d & r3d & r2d & C_ZERO16;
  y <= rcon_enc when (encdec = '0') else
       rcon_dec;
end;
