---------------------------------------------------------------------------
--
-- Copyright (c) 2016
-- Mitsubishi Electric Corporation
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are
-- met:
-- 
-- 1. Redistributions of source code must retain the above copyright
-- notice, this list of conditions and the following disclaimer.
-- 
-- 2. Redistributions in binary form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-- THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-- OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- File Name: MA_minalpher.vhd
-- Type: RTL 
-- Author: Takeshi SUGAWARA
-- Date: 2016/05/26
-- Version: 1.0
-- Abstract:
-- Modification History: 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;  

entity MA_minalpher is
  port(
    Din:            in  std_logic_vector(255 downto 0);
    Kin:            in  std_logic_vector(127 downto 0);
    Nin:            in  std_logic_vector(127 downto 0);  
    Dout:           out std_logic_vector(255 downto 0);
    Tout:           out std_logic_vector(255 downto 0);
    command:        in  std_logic_vector(  4 downto 0);
    CLK:            in  std_logic;
    RSTn:           in  std_logic;
    activate:       in  std_logic;
    Dout_valid:     out std_logic;
    Tout_valid:     out std_logic;
    command_finish: out std_logic
   );
end MA_minalpher;

architecture RTL of MA_minalpher is
  component MA_LFSR_update is
    port(
      Din:    in  std_logic_vector(255 downto 0);
      mode:   in  std_logic;
      Dout:   out std_logic_vector(255 downto 0);
      times2: out std_logic_vector(255 downto 0);
      times3: out std_logic_vector(255 downto 0)
      );
  end component;

  component MA_perm256 is
    port(
      Din:    in  std_logic_vector(255 downto 0);
      Drdy:   in  std_logic;
      RSTn:   in  std_logic;
      CLK:    in  std_logic;
      encdec: in  std_logic;
      Dout:   out std_logic_vector(255 downto 0);
      Dvld:   out std_logic    
      );
  end component;
  
  type state_type is (
    ST_IDLE                ,
    ST_TWEAK               ,
    ST_TAG                 ,
    ST_TAG_FINAL           ,
    ST_DECODE              ,
    ST_ENC_THEN_TAG1       ,
    ST_ENC_THEN_TAG2       ,
    ST_ENC_THEN_TAG3       ,
    ST_ENC_THEN_TAG_FINAL1 ,
    ST_ENC_THEN_TAG_FINAL2 ,
    ST_ENC_THEN_TAG_FINAL3 ,
    ST_DEC_THEN_TAG1       ,
    ST_DEC_THEN_TAG2       ,
    ST_DEC_THEN_TAG3       ,
    ST_DEC_THEN_TAG_FINAL1 ,
    ST_DEC_THEN_TAG_FINAL2 ,
    ST_DEC_THEN_TAG_FINAL3 ,
    ST_DEC_THEN_TAG_FINAL4 ,
    ST_ILLEGAL             );
  
  signal state_reg:    state_type;
  signal command_reg:  std_logic_vector(4 downto 0);  
  signal perm_encdec_reg: std_logic;

  constant COM_KEYSET             : std_logic_vector(4 downto 0) := "00000";
  constant COM_NSET               : std_logic_vector(4 downto 0) := "00001";
  constant COM_TWEAK_ENCMAC       : std_logic_vector(4 downto 0) := "00010";
  constant COM_TWEAK_ENCENC       : std_logic_vector(4 downto 0) := "00011";
  constant COM_TWEAK_MACMAC       : std_logic_vector(4 downto 0) := "00100";
  constant COM_TWEAK_2            : std_logic_vector(4 downto 0) := "00101";
  constant COM_TWEAK_3            : std_logic_vector(4 downto 0) := "00110";
  constant COM_RESET_TAG          : std_logic_vector(4 downto 0) := "00111";
  constant COM_TAG                : std_logic_vector(4 downto 0) := "01000";
  constant COM_TAG_FINAL          : std_logic_vector(4 downto 0) := "01001";
  constant COM_ENC_THEN_TAG       : std_logic_vector(4 downto 0) := "01101";
  constant COM_ENC_THEN_TAG_FINAL : std_logic_vector(4 downto 0) := "01110";
  constant COM_DEC_THEN_TAG       : std_logic_vector(4 downto 0) := "10001";
  constant COM_DEC_THEN_TAG_FINAL : std_logic_vector(4 downto 0) := "10010";

  constant P_MODE_X2: std_logic := '0';
  constant P_MODE_X3: std_logic := '1';  

  signal K_reg:       std_logic_vector(127 downto 0);
  signal N_reg:       std_logic_vector(103 downto 0);
  signal L_reg:       std_logic_vector(255 downto 0);
  signal T_reg:       std_logic_vector(255 downto 0);
  signal Din_reg:     std_logic_vector(255 downto 0);

  signal T_reg_mask:  std_logic_vector(255 downto 0);  
  signal L_update:    std_logic_vector(255 downto 0);
  signal LFSR_times2: std_logic_vector(255 downto 0);
  signal LFSR_times3: std_logic_vector(255 downto 0);
  signal mode_update: std_logic;

  signal perm_Din:    std_logic_vector(255 downto 0);
  signal perm_Dout:   std_logic_vector(255 downto 0);  
  signal perm_Drdy:   std_logic;
  signal perm_Dvld:   std_logic;  

  signal kn_concat:   std_logic_vector(255 downto 0);
  signal L_reg_mask:  std_logic_vector(255 downto 0);
  signal L_reg_in:    std_logic_vector(255 downto 0);  
  signal Dout_tmp:    std_logic_vector(255 downto 0);

  signal perm_Drdy_tmp:     std_logic;
  signal tweak_enable_flag: std_logic;

begin

  LFSR_update: MA_LFSR_update port map(
    Din => L_reg,
    mode => mode_update,
    Dout => L_update, 
    times2 => LFSR_times2,
    times3 => LFSR_times3);
  
  perm256: MA_perm256 port map(
    Din => perm_Din, 
    Drdy => perm_Drdy,
    RSTn => RSTn,
    CLK => CLK,
    encdec => perm_encdec_reg,
    Dout => perm_Dout, 
    Dvld => perm_Dvld);  


  Tout_valid <= '1' when ( ( (perm_Dvld = '1') and (state_reg = ST_TAG_FINAL          ) ) or
                           ( (perm_Dvld = '1') and (state_reg = ST_ENC_THEN_TAG_FINAL3) ) or
                           ( (perm_Dvld = '1') and (state_reg = ST_DEC_THEN_TAG_FINAL4) ) ) else
                '0';
  

  Dout_valid <= '1' when ( (                        state_reg = ST_DEC_THEN_TAG_FINAL2  ) or
                           ( (perm_Dvld = '1') and (state_reg = ST_ENC_THEN_TAG1)       ) or
                           ( (perm_Dvld = '1') and (state_reg = ST_ENC_THEN_TAG_FINAL1) ) or
                           ( (perm_Dvld = '1') and (state_reg = ST_DEC_THEN_TAG1)       ) ) else
                '0';
  
  command_finish <= '1' when ( ( (perm_Dvld = '1') and (state_reg = ST_TWEAK)               ) or
                               ( (perm_Dvld = '1') and (state_reg = ST_TAG)                 ) or
                               ( (perm_Dvld = '1') and (state_reg = ST_TAG_FINAL)           ) or
                               ( (perm_Dvld = '1') and (state_reg = ST_ENC_THEN_TAG3)       ) or
                               ( (perm_Dvld = '1') and (state_reg = ST_DEC_THEN_TAG3)       ) or
                               ( (perm_Dvld = '1') and (state_reg = ST_ENC_THEN_TAG_FINAL3) ) or
                               ( (perm_Dvld = '1') and (state_reg = ST_DEC_THEN_TAG_FINAL4) ) ) else
                    '0';
                    
  Tout <= T_reg;
  T_reg_mask <= T_reg when ( (command_reg = COM_TAG_FINAL       ) or
                             (state_reg = ST_ENC_THEN_TAG_FINAL2) or
                             (state_reg = ST_DEC_THEN_TAG_FINAL3) ) else
                (others => '0');

  mode_update <= P_MODE_X3 when ( (command = COM_TWEAK_3               ) or
                                  (command_reg = COM_ENC_THEN_TAG_FINAL) or
                                  (command_reg = COM_DEC_THEN_TAG_FINAL) ) else
                 P_MODE_X2;
  
  perm_Din <= kn_concat xor L_reg_mask xor T_reg_mask;


  kn_concat_decoder: process(command_reg, K_reg, N_reg, Din_reg)
  begin
    case command_reg is
      when COM_TWEAK_ENCMAC => kn_concat <= K_reg & X"00000000000000000000000000000000";
      when COM_TWEAK_ENCENC => kn_concat <= K_reg & X"400000" & N_reg;
      when COM_TWEAK_MACMAC => kn_concat <= K_reg & X"80000000000000000000000000000000";
      when others =>           kn_concat <= Din_reg;
    end case;
  end process kn_concat_decoder;

  perm_Drdy_tmp <= '1' when ( ( (state_reg = ST_DECODE) and (command_reg = COM_TWEAK_ENCMAC)       ) or
                              ( (state_reg = ST_DECODE) and (command_reg = COM_TWEAK_ENCENC)       ) or
                              ( (state_reg = ST_DECODE) and (command_reg = COM_TWEAK_MACMAC)       ) or
                              ( (state_reg = ST_DECODE) and (command_reg = COM_ENC_THEN_TAG)       ) or
                              ( (state_reg = ST_DECODE) and (command_reg = COM_ENC_THEN_TAG_FINAL) ) or
                              ( (state_reg = ST_DECODE) and (command_reg = COM_DEC_THEN_TAG)       ) or
                              ( (state_reg = ST_DECODE) and (command_reg = COM_DEC_THEN_TAG_FINAL) ) or
                              ( (state_reg = ST_DECODE) and (command_reg = COM_TAG)                ) or
                              ( (state_reg = ST_DECODE) and (command_reg = COM_TAG_FINAL)          ) ) else
                   '0';

  perm_Drdy  <= '1' when ( (perm_Drdy_tmp = '1'                ) or
                           (state_reg = ST_ENC_THEN_TAG2       ) or
                           (state_reg = ST_DEC_THEN_TAG2       ) or
                           (state_reg = ST_ENC_THEN_TAG_FINAL2 ) or
                           (state_reg = ST_DEC_THEN_TAG_FINAL3 ) ) else
                '0';

  L_reg_in <= L_update when ( ( (state_reg = ST_IDLE) and (command = COM_TWEAK_2) ) or
                              ( (state_reg = ST_IDLE) and (command = COM_TWEAK_3) ) ) else
              (perm_Dout xor kn_concat);

  tweak_enable_flag <= '1' when ( (command_reg = COM_ENC_THEN_TAG      ) or 
                                  (command_reg = COM_ENC_THEN_TAG_FINAL) or 
                                  (command_reg = COM_DEC_THEN_TAG      ) or 
                                  (command_reg = COM_DEC_THEN_TAG_FINAL) or 
                                  (command_reg = COM_TAG               ) or
                                  (command_reg = COM_TAG_FINAL         ) ) else
                       '0';
  L_reg_mask  <= L_reg when (tweak_enable_flag = '1') else (others => '0');
  Dout_tmp <= perm_Dout xor L_reg_mask;
  Dout <= Dout_tmp;

  proc_encdec_reg: process(CLK)
  begin
    if(CLK'event and CLK='1') then
      if RSTn = '0' then
        perm_encdec_reg <= '0';
      else -- if RSTn = 1
        if(state_reg = ST_IDLE) then
          if(activate = '1') then
            case command is
              when COM_DEC_THEN_TAG =>       perm_encdec_reg <= '1';
              when COM_DEC_THEN_TAG_FINAL => perm_encdec_reg <= '1';
              when others =>                 perm_encdec_reg <= '0';
            end case;
          end if;
        elsif( (state_reg = ST_DEC_THEN_TAG1) and (perm_Dvld = '1')) then
          perm_encdec_reg <= '0';
        elsif(state_reg = ST_DEC_THEN_TAG_FINAL2) then
          perm_encdec_reg <= '0';
        end if;
      end if;
    end if;      
  end process proc_encdec_reg;

  proc_T_reg: process(CLK)
  begin
    if(CLK'event and CLK='1') then
      if RSTn = '0' then
        T_reg <= (others => '0');
      else
        if((state_reg = ST_IDLE) and
           (activate = '1') and
           (command = COM_RESET_TAG)) then
          T_reg <= (others => '0');
        elsif(perm_Dvld = '1') then
          if( (state_reg = ST_TAG_FINAL) or
              (state_reg = ST_ENC_THEN_TAG_FINAL3) or
              (state_reg = ST_DEC_THEN_TAG_FINAL4) ) then
            T_reg <= Dout_tmp;
          elsif( (state_reg = ST_TAG) or
                 (state_reg = ST_ENC_THEN_TAG3) or
                 (state_reg = ST_DEC_THEN_TAG3) ) then
            T_reg <= Dout_tmp xor T_reg;
          end if;
        end if;
      end if;
    end if;
  end process proc_T_reg;

  proc_L_reg: process(CLK)
  begin
    if(CLK'event and CLK='1') then
      if RSTn = '0' then
        L_reg <= (others => '0');
      else
        if((state_reg = ST_IDLE) and
           (activate = '1') and
           ( (command = COM_TWEAK_2) or (command = COM_TWEAK_3) )) then
          L_reg <= L_reg_in;
        elsif(state_reg = ST_DEC_THEN_TAG_FINAL2) then
          L_reg <= L_update;
        elsif(perm_Dvld = '1') then
          if(state_reg = ST_TWEAK) then
            L_reg <= L_reg_in;
          elsif( (state_reg = ST_ENC_THEN_TAG1) or
                 (state_reg = ST_DEC_THEN_TAG1) or
                 (state_reg = ST_ENC_THEN_TAG3) or
                 (state_reg = ST_DEC_THEN_TAG3) or
                 (state_reg = ST_ENC_THEN_TAG_FINAL1) ) then
            L_reg <= L_update;
          end if;
        end if;
      end if;
    end if;
  end process proc_L_reg;

  proc_K_reg: process(CLK)
  begin
    if(CLK'event and CLK='1') then
      if RSTn = '0' then
      else
        if((state_reg = ST_IDLE) and
           (activate = '1') and
           (command = COM_KEYSET)) then
          K_reg <= Kin;
        end if;
      end if;
    end if;
  end process proc_K_reg;

  proc_N_reg: process(CLK)
  begin
    if(CLK'event and CLK='1') then
      if RSTn = '0' then
      else
        if((state_reg = ST_IDLE) and
           (activate = '1') and
           (command = COM_NSET)) then
          N_reg <= Nin(103 downto 0);
        end if;
      end if;
    end if;
  end process proc_N_reg;
  
  proc_state_reg: process(CLK)
  begin
    if(CLK'event and CLK='1') then
      if RSTn = '0' then
        state_reg <= ST_IDLE;
      else
        case state_reg is
          when ST_IDLE =>
            if(activate = '1') then
              case command is
                when COM_KEYSET    => state_reg <= ST_IDLE;
                when COM_NSET      => state_reg <= ST_IDLE;
                when COM_RESET_TAG => state_reg <= ST_IDLE;
                when COM_TWEAK_2   => state_reg <= ST_IDLE;
                when COM_TWEAK_3   => state_reg <= ST_IDLE;
                when others =>
                  command_reg <= command;
                  state_reg <= ST_DECODE;
                  Din_reg <= Din;
              end case;
            end if;
            
          when ST_DECODE =>
            case command_reg is
              when COM_TWEAK_ENCMAC       => state_reg <= ST_TWEAK;
              when COM_TWEAK_ENCENC       => state_reg <= ST_TWEAK;
              when COM_TWEAK_MACMAC       => state_reg <= ST_TWEAK;
              when COM_TAG                => state_reg <= ST_TAG;
              when COM_TAG_FINAL          => state_reg <= ST_TAG_FINAL;
              when COM_ENC_THEN_TAG       => state_reg <= ST_ENC_THEN_TAG1;
              when COM_ENC_THEN_TAG_FINAL => state_reg <= ST_ENC_THEN_TAG_FINAL1;
              when COM_DEC_THEN_TAG       => state_reg <= ST_DEC_THEN_TAG1;
              when COM_DEC_THEN_TAG_FINAL => state_reg <= ST_DEC_THEN_TAG_FINAL1;
              when others                 => state_reg <= ST_ILLEGAL;
            end case;
            
          when ST_TWEAK | ST_TAG | ST_TAG_FINAL =>
            if(perm_Dvld = '1') then
              state_reg <= ST_IDLE;
            end if;

          when ST_ENC_THEN_TAG1 => 
            if(perm_Dvld = '1') then
              state_reg <= ST_ENC_THEN_TAG2;
              Din_reg <= Dout_tmp;
            end if;
            
          when ST_ENC_THEN_TAG2 =>                          state_reg <= ST_ENC_THEN_TAG3;
          when ST_ENC_THEN_TAG3 => if(perm_Dvld = '1') then state_reg <= ST_IDLE;          end if;
          when ST_DEC_THEN_TAG1 => if(perm_Dvld = '1') then state_reg <= ST_DEC_THEN_TAG2; end if;
          when ST_DEC_THEN_TAG2 =>                          state_reg <= ST_DEC_THEN_TAG3; 
          when ST_DEC_THEN_TAG3 => if(perm_Dvld = '1') then state_reg <= ST_IDLE;          end if;

          when ST_ENC_THEN_TAG_FINAL1 =>
            if(perm_Dvld = '1') then
              state_reg <= ST_ENC_THEN_TAG_FINAL2;
              Din_reg <= Dout_tmp;
            end if;

          when ST_ENC_THEN_TAG_FINAL2 =>                          state_reg <= ST_ENC_THEN_TAG_FINAL3;
          when ST_ENC_THEN_TAG_FINAL3 => if(perm_Dvld = '1') then state_reg <= ST_IDLE; end if;
          when ST_DEC_THEN_TAG_FINAL1 => if(perm_Dvld = '1') then state_reg <= ST_DEC_THEN_TAG_FINAL2; end if;
          when ST_DEC_THEN_TAG_FINAL2 =>                          state_reg <= ST_DEC_THEN_TAG_FINAL3;                                         
          when ST_DEC_THEN_TAG_FINAL3 =>                          state_reg <= ST_DEC_THEN_TAG_FINAL4;
          when ST_DEC_THEN_TAG_FINAL4 => if(perm_Dvld = '1') then state_reg <= ST_IDLE; end if;

          when others => -- do nothing
        end case;
      end if;
    end if;
  end process proc_state_reg;
end;
