---------------------------------------------------------------------------
--
-- Copyright (c) 2016
-- Mitsubishi Electric Corporation
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are
-- met:
-- 
-- 1. Redistributions of source code must retain the above copyright
-- notice, this list of conditions and the following disclaimer.
-- 
-- 2. Redistributions in binary form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-- THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-- OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- File Name: MA_mx128.vhd
-- Type: RTL 
-- Author: Takeshi SUGAWARA
-- Date: 2016/05/26
-- Version: 1.0
-- Abstract:
-- Modification History: 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;  

entity MA_mx128 is
  port(
    x: in  std_logic_vector(127 downto 0);
    y: out std_logic_vector(127 downto 0)
    );
end MA_mx128;

architecture RTL of MA_mx128 is
  component MA_mx16 is
  port(
    x00: in std_logic_vector(3 downto 0);
    x01: in std_logic_vector(3 downto 0);
    x02: in std_logic_vector(3 downto 0);
    x03: in std_logic_vector(3 downto 0);
    y00: out std_logic_vector(3 downto 0);
    y01: out std_logic_vector(3 downto 0);
    y02: out std_logic_vector(3 downto 0);
    y03: out std_logic_vector(3 downto 0)
    );
  end component;

  signal x00, x01, x02, x03: std_logic_vector(3 downto 0);
  signal x04, x05, x06, x07: std_logic_vector(3 downto 0);  
  signal x08, x09, x10, x11: std_logic_vector(3 downto 0);
  signal x12, x13, x14, x15: std_logic_vector(3 downto 0);  
  signal x16, x17, x18, x19: std_logic_vector(3 downto 0);
  signal x20, x21, x22, x23: std_logic_vector(3 downto 0);  
  signal x24, x25, x26, x27: std_logic_vector(3 downto 0);
  signal x28, x29, x30, x31: std_logic_vector(3 downto 0);  

  signal y00, y01, y02, y03: std_logic_vector(3 downto 0);
  signal y04, y05, y06, y07: std_logic_vector(3 downto 0);  
  signal y08, y09, y10, y11: std_logic_vector(3 downto 0);
  signal y12, y13, y14, y15: std_logic_vector(3 downto 0);  
  signal y16, y17, y18, y19: std_logic_vector(3 downto 0);
  signal y20, y21, y22, y23: std_logic_vector(3 downto 0);  
  signal y24, y25, y26, y27: std_logic_vector(3 downto 0);
  signal y28, y29, y30, y31: std_logic_vector(3 downto 0);  
begin
  x00 <= x(127 downto 124);
  x01 <= x(123 downto 120);
  x02 <= x(119 downto 116);
  x03 <= x(115 downto 112);
  x04 <= x(111 downto 108);
  x05 <= x(107 downto 104);
  x06 <= x(103 downto 100);
  x07 <= x( 99 downto  96);
  x08 <= x( 95 downto  92);
  x09 <= x( 91 downto  88);
  x10 <= x( 87 downto  84);
  x11 <= x( 83 downto  80);
  x12 <= x( 79 downto  76);
  x13 <= x( 75 downto  72);
  x14 <= x( 71 downto  68);
  x15 <= x( 67 downto  64);
  x16 <= x( 63 downto  60);
  x17 <= x( 59 downto  56);
  x18 <= x( 55 downto  52);
  x19 <= x( 51 downto  48);
  x20 <= x( 47 downto  44);
  x21 <= x( 43 downto  40);
  x22 <= x( 39 downto  36);
  x23 <= x( 35 downto  32);
  x24 <= x( 31 downto  28);
  x25 <= x( 27 downto  24);
  x26 <= x( 23 downto  20);
  x27 <= x( 19 downto  16);
  x28 <= x( 15 downto  12);
  x29 <= x( 11 downto   8);
  x30 <= x(  7 downto   4);
  x31 <= x(  3 downto   0);

  y <= y00 & y01 & y02 & y03 & y04 & y05 & y06 & y07 &
       y08 & y09 & y10 & y11 & y12 & y13 & y14 & y15 &
       y16 & y17 & y18 & y19 & y20 & y21 & y22 & y23 &
       y24 & y25 & y26 & y27 & y28 & y29 & y30 & y31;
  
 mx00: MA_mx16 port map ( x00 => x00, x01 => x08, x02 => x16, x03 => x24,
                          y00 => y00, y01 => y08, y02 => y16, y03 => y24);
 mx01: MA_mx16 port map ( x00 => x01, x01 => x09, x02 => x17, x03 => x25,
                          y00 => y01, y01 => y09, y02 => y17, y03 => y25);
 mx02: MA_mx16 port map ( x00 => x02, x01 => x10, x02 => x18, x03 => x26,
                          y00 => y02, y01 => y10, y02 => y18, y03 => y26);
 mx03: MA_mx16 port map ( x00 => x03, x01 => x11, x02 => x19, x03 => x27,
                          y00 => y03, y01 => y11, y02 => y19, y03 => y27);
 mx04: MA_mx16 port map ( x00 => x04, x01 => x12, x02 => x20, x03 => x28,
                          y00 => y04, y01 => y12, y02 => y20, y03 => y28);  
 mx05: MA_mx16 port map ( x00 => x05, x01 => x13, x02 => x21, x03 => x29,
                          y00 => y05, y01 => y13, y02 => y21, y03 => y29);
 mx06: MA_mx16 port map ( x00 => x06, x01 => x14, x02 => x22, x03 => x30,
                          y00 => y06, y01 => y14, y02 => y22, y03 => y30);  
 mx07: MA_mx16 port map ( x00 => x07, x01 => x15, x02 => x23, x03 => x31,
                          y00 => y07, y01 => y15, y02 => y23, y03 => y31);  
end;

