---------------------------------------------------------------------------
--
-- Copyright (c) 2016
-- Mitsubishi Electric Corporation
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are
-- met:
-- 
-- 1. Redistributions of source code must retain the above copyright
-- notice, this list of conditions and the following disclaimer.
-- 
-- 2. Redistributions in binary form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-- THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-- OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- File Name: MA_perm256.vhd
-- Type: RTL 
-- Author: Takeshi SUGAWARA
-- Date: 2016/05/26
-- Version: 1.0
-- Abstract:
-- Modification History: 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;  

entity MA_perm256 is
  port(
    Din:    in  std_logic_vector(255 downto 0);
    Drdy:   in  std_logic;
    RSTn:   in  std_logic;
    CLK:    in  std_logic;
    encdec: in  std_logic;
    Dout:   out std_logic_vector(255 downto 0);
    Dvld:   out std_logic    
    );
end MA_perm256;

architecture RTL of MA_perm256 is
  component MA_const_gen256 is
  port(
    rnd:    in  std_logic_vector(17 downto 0);
    encdec: in  std_logic;
    y:      out std_logic_vector(255 downto 0)
    );
  end component;

  component MA_roundfunc is
    port(
      x:    in  std_logic_vector(255 downto 0);
      rcon: in  std_logic_vector(255 downto 0);    
      y:    out std_logic_vector(255 downto 0);
      Dout: out std_logic_vector(255 downto 0) 
      );
  end component;
  
  constant C_RND_INIT: std_logic_vector(17 downto 0)
    := "000000000000000001";

  signal data_reg:  std_logic_vector(255 downto 0);
  signal rnd_reg:   std_logic_vector(17 downto 0);
  signal busy_reg:  std_logic;

  signal rcon:      std_logic_vector(255 downto 0);
  signal Dnext:     std_logic_vector(255 downto 0);
  signal rnd_next:  std_logic_vector(17 downto 0);
  signal is_final:  std_logic;
begin

  cg256: MA_const_gen256 port map (
    rnd => rnd_reg,
    encdec => encdec,
    y => rcon);

  roundfunc: MA_roundfunc port map (
    x => data_reg,
    rcon => rcon,
    y => Dnext,
    Dout => Dout);
  
  is_final <= '1' when rnd_reg(17) = '1' else
                    '0';
  Dvld <= is_final;

  rnd_next(17 downto 1) <= rnd_reg(16 downto 0);
  rnd_next(0) <= rnd_reg(17);
  
  seq: process(CLK)
  begin
    if(CLK'event and CLK='1') then        
      if RSTn = '0' then
        rnd_reg <= C_RND_INIT;
        busy_reg  <= '0';
      else -- if RSTn == 1'b1
        if(busy_reg = '0') then
          if(Drdy = '1') then
            data_reg <= Din;
            rnd_reg <= C_RND_INIT;
            busy_reg <= '1';
          end if;
        else -- if busy_reg == 1'b1
          if(is_final = '1') then -- final round
            rnd_reg <= C_RND_INIT;
            if(Drdy = '1') then -- next iteration
              data_reg <= Din;
            else 
              busy_reg <= '0';
            end if;
          else -- non-final round
            rnd_reg <= rnd_next;
            data_reg <= Dnext;
          end if;
        end if;
      end if;
    end if;
  end process seq;
end;
