---------------------------------------------------------------------------
--
-- Copyright (c) 2016
-- Mitsubishi Electric Corporation
-- All rights reserved.
-- 
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions are
-- met:
-- 
-- 1. Redistributions of source code must retain the above copyright
-- notice, this list of conditions and the following disclaimer.
-- 
-- 2. Redistributions in binary form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
-- 
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-- DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-- THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-- (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-- OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
--
-- File Name: MA_sr128.vhd
-- Type: RTL 
-- Author: Takeshi SUGAWARA
-- Date: 2016/05/26
-- Version: 1.0
-- Abstract:
-- Modification History: 
--
---------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;  

entity MA_sr128 is
  port(
    x: in  std_logic_vector(127 downto 0);
    y: out std_logic_vector(127 downto 0)
    );
end MA_sr128;

architecture RTL of MA_sr128 is
  component MA_sr1_32 is
    port(
      x: in  std_logic_vector(31 downto 0);
      y: out std_logic_vector(31 downto 0)
      );
  end component;

  component MA_sr2_32 is
    port(
      x: in  std_logic_vector(31 downto 0);
      y: out std_logic_vector(31 downto 0)
      );
  end component;

  component MA_sr1inv_32 is
    port(
      x: in  std_logic_vector(31 downto 0);
      y: out std_logic_vector(31 downto 0)
      );
  end component;

  component MA_sr2inv_32 is
    port(
      x: in  std_logic_vector(31 downto 0);
      y: out std_logic_vector(31 downto 0)
      );
  end component; 
 
  signal x_row0: std_logic_vector(31 downto 0);
  signal x_row1: std_logic_vector(31 downto 0);
  signal x_row2: std_logic_vector(31 downto 0);
  signal x_row3: std_logic_vector(31 downto 0);

  signal y_row0: std_logic_vector(31 downto 0);
  signal y_row1: std_logic_vector(31 downto 0);
  signal y_row2: std_logic_vector(31 downto 0);
  signal y_row3: std_logic_vector(31 downto 0);
begin
  x_row0 <= x(127 downto 96);
  x_row1 <= x( 95 downto 64);
  x_row2 <= x( 63 downto 32);
  x_row3 <= x( 31 downto  0);

  y <= y_row0 & y_row1 & y_row2 & y_row3;

  sr1f: MA_sr1_32    port map ( x => x_row0, y => y_row0);
  sr2f: MA_sr2_32    port map ( x => x_row1, y => y_row1);
  sr1i: MA_sr1inv_32 port map ( x => x_row2, y => y_row2);
  sr2i: MA_sr2inv_32 port map ( x => x_row3, y => y_row3);  
end;
